/////////////////////////////////////////////////
//
//  filename: DispWindow.cpp
//  author:Chafumi Touji
//  date: 2021/06/30 ver1.00
//
/////////////////////////////////////////////////

#include "defcha3d.h"

CDispWindow::CDispWindow() {

	Width = 0;
	Height = 0;
	IsFullScreen = false;
	BackFullScreen = false;
	IsTransparent = false;

	ResolutionWidth = FULLHD_WIDTH;
	ResolutionHeight = FULLHD_HEIGHT;

	UpdateFrameCount = 1;
	FPS = 0.0;

	AppExit = false;
}

CDispWindow::~CDispWindow() {
}

bool CDispWindow::CreateWnd(HWND ParentWnd, int x, int y, int w, int h) {

	WNDCLASSEX wcx;
	memset(&wcx, 0, sizeof(WNDCLASSEX));
	wcx.style = CS_HREDRAW | CS_VREDRAW;
	wcx.cbSize = sizeof(WNDCLASSEX);
	wcx.hInstance = NULL;
	wcx.hIcon = NULL;
	wcx.lpszMenuName = NULL;
	wcx.hCursor = LoadCursor(NULL, IDC_ARROW);
	wcx.lpfnWndProc = &CDispWindow::WindowProc;
	wcx.lpszClassName = L"CLASS_DISPWINDOW";
	RegisterClassEx(&wcx);

	Width = w;
	Height = h;
	m_hWnd = CreateWindowEx(WS_EX_LEFT, L"CLASS_DISPWINDOW", _T("ModelViewer"), WS_CHILD, x, y, w, h, ParentWnd, NULL, NULL, NULL);
	::SetWindowLongPtr(m_hWnd, GWLP_USERDATA, (LONG_PTR)this);

	ResolutionWidth = FULLHD_WIDTH;
	ResolutionHeight = FULLHD_HEIGHT;

	Device = new CCommonDevice();
	Device->CreateCommonDevice(m_hWnd, ResolutionWidth, ResolutionHeight );

	ObjectManager = new CObjectManager(Device, this);

	Camera = new CCameraBase( ResolutionWidth, ResolutionHeight );
	Camera->LightPosX = -1500.0f;
	Camera->LightPosY = 1500.0f;
	Camera->LightPosZ = 1500.0f;

	/////////////////////////////////////////////////////

	::ShowWindow(m_hWnd, SW_SHOW);

	//////////////////////////////////////////////////

	return true;
}

unsigned _stdcall CDispWindow::CallbackProc1(void* Param) {

	CDispWindow* disp_wnd = (CDispWindow*)Param;

	while (disp_wnd->AppExit == false ) {

		SYSTEMTIME st;
		::GetSystemTime(&st);
		unsigned int  t = (unsigned int)(st.wSecond * 1000 + st.wMilliseconds);
		static unsigned int prev_time = 0;
		static unsigned int count_sec = 0;
		static int count_frame = 0;
		if (prev_time > t)
			prev_time -= 60 * 1000;
		unsigned int interval = t - prev_time;
		count_sec += interval;
		prev_time = t;
		count_frame += 1;
		if (count_sec > 1000) {
			disp_wnd->FPS = (float)count_frame * 1000.0f / (float)count_sec;
			count_frame = 0;
			count_sec = 0;
		}

		disp_wnd->ObjectManager->PreAction();
		disp_wnd->ObjectManager->UpdateFrameCount(disp_wnd->UpdateFrameCount);
		disp_wnd->ObjectManager->UpdateTransform();
		disp_wnd->ObjectManager->Action();
		disp_wnd->Camera->SetCamera(disp_wnd->Device);
		disp_wnd->Camera->SetCameraforShadowMap(disp_wnd->Device);

		// R}hAP[^ƃR}hXgZbg.
		disp_wnd->Device->m_pCmdAlloc1->Reset();
		disp_wnd->Device->m_pCmdList1->Reset(disp_wnd->Device->m_pCmdAlloc1, nullptr);

		RenderProc1((void*)disp_wnd);
		RenderProc2((void*)disp_wnd);
		RenderProc3((void*)disp_wnd);

		// R}h̋L^I
		disp_wnd->Device->m_pCmdList1->Close();

		// R}hs.
		ID3D12CommandList* ppCmdLists[] = { disp_wnd->Device->m_pCmdList1 };
		disp_wnd->Device->m_pCmdQue->ExecuteCommandLists(_countof(ppCmdLists), ppCmdLists);

		// \.
		disp_wnd->Device->m_pSwapChain->Present(0, 0);

		// R}h̊ҋ@.
		disp_wnd->WaitForGPU();
	}

	return true;
}

unsigned _stdcall CDispWindow::CallbackProc2(void* Param) {

	CDispWindow* disp_wnd = (CDispWindow*)Param;

	while (disp_wnd->AppExit == false) {

		BYTE key[256];
		ZeroMemory(key, sizeof(key));
		HRESULT hr = disp_wnd->Device->lpKeyboard->GetDeviceState(sizeof(key), key);
		if (FAILED(hr)) {
			// sȂĊJĂx擾
			disp_wnd->Device->lpKeyboard->Acquire();
			disp_wnd->Device->lpKeyboard->GetDeviceState(sizeof(key), key);
		}
		else
			disp_wnd->ObjectManager->OnKeyDown(key);

		Sleep(16);
	}

	return true;
}

bool CDispWindow::Run() {

	AppExit = false;
	UINT thread_id = 1;
	m_hThread1 = (HANDLE) _beginthreadex(NULL, 0, &CallbackProc1, (void*)this, 0, &thread_id);
	m_hThread2 = (HANDLE)_beginthreadex(NULL, 0, &CallbackProc2, (void*)this, 0, &thread_id);

	return true;
}

LRESULT CDispWindow::WindowProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam) {

	CDispWindow* disp_wnd = (CDispWindow*) ::GetWindowLongPtr(hwnd, GWLP_USERDATA);
	if (disp_wnd != NULL)
		disp_wnd->DispWindowProc(hwnd, message, wParam, lParam);

	return DefWindowProc(hwnd, message, wParam, lParam);
}

LRESULT CDispWindow::DispWindowProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam) {

	int width = 0;
	int height = 0;
	int x = 0;
	int y = 0;
	int delta = 0;
	int key = 0;

	switch (message) {
	case WM_PAINT:
		break;
	default:
		break;
	}

	return 0;
}

void CDispWindow::OnSize(UINT nType, int cx, int cy) {

	::InvalidateRect(m_hWnd, NULL, false);
	::UpdateWindow(m_hWnd);
}

bool CDispWindow::CheckCollision(CObjectManager* manager) {

	for (int i = 1; i <= manager->CharacterList->GetSize(); i++) {
		CCharacterBase* obj = (CCharacterBase*)((CListDataA*)manager->CharacterList->GetAt(i))->Data;
		obj->CollisionZ = false;
		obj->CollisionXY = false;
	}
	for (int i = 1; i <= manager->ObjectList->GetSize(); i++) {
		CCharacterBase* obj = (CCharacterBase*)((CListDataA*)manager->ObjectList->GetAt(i))->Data;
		obj->CollisionZ = false;
		obj->CollisionXY = false;
	}

	for (int i = 1; i <= manager->CharacterList->GetSize(); i++) {
		CCharacterBase* obj = (CCharacterBase*)((CListDataA*)manager->CharacterList->GetAt(i))->Data;
		if( obj->CollisionCheck == true )
			obj->CheckCollision();
	}

	return true;
}

bool CDispWindow::DrawCollider(bool draw) {

	if( ObjectManager != NULL )
		ObjectManager->DrawCollider = draw;

	return true;
}

unsigned _stdcall CDispWindow::RenderProc1(void* param){

	CDispWindow* disp_wnd = (CDispWindow*)param;

	disp_wnd->Device->m_pCmdList1->SetDescriptorHeaps(1, &disp_wnd->Device->m_pHeap);
	disp_wnd->Device->m_pCmdList1->SetGraphicsRootSignature(disp_wnd->Device->RootSignature);
	disp_wnd->Device->m_pCmdList1->SetGraphicsRootConstantBufferView(0, disp_wnd->Device->m_pConstantBuffer->GetGPUVirtualAddress());

	// r[|[g̐ݒ.
	D3D12_VIEWPORT vp;
	vp.Width = (float)disp_wnd->ResolutionWidth * 2.0f;
	vp.Height = (float)disp_wnd->ResolutionHeight * 2.0f;
	vp.TopLeftX = 0;
	vp.TopLeftY = 0;
	vp.MinDepth = 0.0f;
	vp.MaxDepth = 1.0f;
	disp_wnd->Device->m_pCmdList1->RSSetViewports(1, &vp);

	// VU[`̐ݒ.
	D3D12_RECT rect;
	rect.top = 0;
	rect.left = 0;
	rect.right = (LONG) (disp_wnd->ResolutionWidth * 2.0f);
	rect.bottom = (LONG) (disp_wnd->ResolutionHeight * 2.0f );
	disp_wnd->Device->m_pCmdList1->RSSetScissorRects(1, &rect);

	// _[^[Qbg̃nh擾.
	D3D12_CPU_DESCRIPTOR_HANDLE handle_rtv = disp_wnd->Device->m_pTmpHeap->GetCPUDescriptorHandleForHeapStart();
	D3D12_CPU_DESCRIPTOR_HANDLE handle_dsv = disp_wnd->Device->m_pDepthHeap->GetCPUDescriptorHandleForHeapStart();

	disp_wnd->Device->m_pCmdList1->SetDescriptorHeaps(1, &disp_wnd->Device->m_pHeap);

	disp_wnd->DrawShadowMap(disp_wnd->ObjectManager, handle_rtv, handle_dsv);

	return true;
}

unsigned _stdcall CDispWindow::RenderProc2(void* param) {

	CDispWindow* disp_wnd = (CDispWindow*)param;

	///////////////////////////////////////////////////////////////////////////

	disp_wnd->Device->m_pCmdList1->SetDescriptorHeaps(1, &disp_wnd->Device->m_pHeap);
	disp_wnd->Device->m_pCmdList1->SetGraphicsRootSignature(disp_wnd->Device->RootSignature);
	disp_wnd->Device->m_pCmdList1->SetGraphicsRootConstantBufferView(0, disp_wnd->Device->m_pConstantBuffer->GetGPUVirtualAddress());

	// r[|[g̐ݒ.
	D3D12_VIEWPORT vp;
	vp.Width = (float)disp_wnd->ResolutionWidth;
	vp.Height = (float)disp_wnd->ResolutionHeight;
	vp.TopLeftX = (float)0;
	vp.TopLeftY = (float)0;
	vp.MinDepth = 0.0f;
	vp.MaxDepth = 1.0f;
	disp_wnd->Device->m_pCmdList1->RSSetViewports(1, &vp);

	// VU[`̐ݒ.
	D3D12_RECT rect;
	rect.top = 0;
	rect.left = 0;
	rect.right = disp_wnd->ResolutionWidth;
	rect.bottom = disp_wnd->ResolutionHeight;
	disp_wnd->Device->m_pCmdList1->RSSetScissorRects(1, &rect);

	// _[^[Qbg̃nh擾.
	D3D12_CPU_DESCRIPTOR_HANDLE handle_rtv = disp_wnd->Device->m_pTmpHeap->GetCPUDescriptorHandleForHeapStart();
	D3D12_CPU_DESCRIPTOR_HANDLE handle_dsv = disp_wnd->Device->m_pDepthHeap->GetCPUDescriptorHandleForHeapStart();

	disp_wnd->Device->m_pCmdList1->SetDescriptorHeaps(1, &disp_wnd->Device->m_pHeap);

	//////////////////////////////////////////////////////////////////

	disp_wnd->Device->m_pCmdList1->SetGraphicsRootDescriptorTable(6, disp_wnd->Device->ResManager->GetGPUHandle(disp_wnd->Device->ShadowMapSRVNum));

	disp_wnd->Draw(disp_wnd->ObjectManager, handle_rtv, handle_dsv);

	return true;
}

unsigned _stdcall CDispWindow::RenderProc3(void* param) {

	CDispWindow* disp_wnd = (CDispWindow*)param;

	disp_wnd->Device->m_pCmdList1->SetDescriptorHeaps(1, &disp_wnd->Device->m_pHeap);
	disp_wnd->Device->m_pCmdList1->SetGraphicsRootSignature(disp_wnd->Device->RootSignature);
	disp_wnd->Device->m_pCmdList1->SetGraphicsRootConstantBufferView(0, disp_wnd->Device->m_pConstantBuffer->GetGPUVirtualAddress());

	// r[|[g̐ݒ.
	D3D12_VIEWPORT vp;
	vp.Width = (float)disp_wnd->ResolutionWidth;
	vp.Height = (float)disp_wnd->ResolutionHeight;
	vp.TopLeftX = (float)0;
	vp.TopLeftY = (float)0;
	vp.MinDepth = 0.0f;
	vp.MaxDepth = 1.0f;
	disp_wnd->Device->m_pCmdList1->RSSetViewports(1, &vp);

	// VU[`̐ݒ.
	D3D12_RECT rect;
	rect.top = 0;
	rect.left = 0;
	rect.right = disp_wnd->ResolutionWidth;
	rect.bottom = disp_wnd->ResolutionHeight;
	disp_wnd->Device->m_pCmdList1->RSSetScissorRects(1, &rect);

	// _[^[Qbg̃nh擾.
	disp_wnd->Device->m_pCmdList1->SetDescriptorHeaps(1, &disp_wnd->Device->m_pHeap);

	///////////////////////////////////////////////////////////////////////////
	//_Oʂ

	disp_wnd->Device->m_pCmdList1->SetPipelineState(disp_wnd->Device->m_pPipeLineState2);

	disp_wnd->Device->m_pCmdList1->SetGraphicsRootDescriptorTable(5, disp_wnd->Device->ResManager->GetGPUHandle(disp_wnd->Device->TmpSRVNum));
	disp_wnd->Device->m_pCmdList1->SetGraphicsRootDescriptorTable(7, disp_wnd->Device->ResManager->GetGPUHandle(disp_wnd->Device->ToonSRVNum));
	disp_wnd->Device->m_pCmdList1->SetGraphicsRootDescriptorTable(8, disp_wnd->Device->ResManager->GetGPUHandle(disp_wnd->Device->ToonDepthSRVNum));
	disp_wnd->Device->m_pCmdList1->SetGraphicsRootDescriptorTable(9, disp_wnd->Device->ResManager->GetGPUHandle(disp_wnd->Device->ColliderSRVNum));

	D3D12_CPU_DESCRIPTOR_HANDLE handle_rtv = disp_wnd->Device->m_pRTVHeap->GetCPUDescriptorHandleForHeapStart();
	D3D12_CPU_DESCRIPTOR_HANDLE handle_dsv = disp_wnd->Device->m_pDepthHeap->GetCPUDescriptorHandleForHeapStart();
	handle_rtv.ptr += (disp_wnd->Device->FrameIndex * disp_wnd->Device->RTVDescriptorSize);

	D3D12_RESOURCE_BARRIER barrier;
	ZeroMemory(&barrier, sizeof(barrier));
	barrier.Type = D3D12_RESOURCE_BARRIER_TYPE_TRANSITION;
	barrier.Flags = D3D12_RESOURCE_BARRIER_FLAG_NONE;
	barrier.Transition.pResource = disp_wnd->Device->m_pRenderTarget[disp_wnd->Device->FrameIndex];
	barrier.Transition.StateBefore = D3D12_RESOURCE_STATE_PRESENT;
	barrier.Transition.StateAfter = D3D12_RESOURCE_STATE_RENDER_TARGET;
	barrier.Transition.Subresource = D3D12_RESOURCE_BARRIER_FLAG_NONE;
	disp_wnd->Device->m_pCmdList1->ResourceBarrier(1, &barrier);

	disp_wnd->Device->m_pCmdList1->IASetPrimitiveTopology(D3D_PRIMITIVE_TOPOLOGY_TRIANGLESTRIP);
	disp_wnd->Device->m_pCmdList1->IASetVertexBuffers(0, 1, disp_wnd->Device->VertexBufferView);

	// _[^[Qbg̐ݒ.
	disp_wnd->Device->m_pCmdList1->OMSetRenderTargets(1, &handle_rtv, FALSE, &handle_dsv);

	// _[^[Qbgr[NA.
	const float clearColor2[] = { 0, 0, 0, 0 };
	disp_wnd->Device->m_pCmdList1->ClearRenderTargetView(handle_rtv, clearColor2, 0, nullptr);
	disp_wnd->Device->m_pCmdList1->ClearDepthStencilView(handle_dsv, D3D12_CLEAR_FLAG_DEPTH, 1.0f, 0, 0, nullptr);

	disp_wnd->Device->m_pCmdList1->IASetPrimitiveTopology(D3D_PRIMITIVE_TOPOLOGY_TRIANGLESTRIP);
	disp_wnd->Device->m_pCmdList1->IASetVertexBuffers(0, 1, disp_wnd->Device->VertexBufferView);

	disp_wnd->Device->m_pCmdList1->DrawInstanced(4, 1, 0, 0);

	// \[XoA̐ݒ.
	barrier.Transition.StateBefore = D3D12_RESOURCE_STATE_RENDER_TARGET;
	barrier.Transition.StateAfter = D3D12_RESOURCE_STATE_PRESENT;
	disp_wnd->Device->m_pCmdList1->ResourceBarrier(1, &barrier);

	return true;
}

void CDispWindow::WaitForGPU() {

	// VOiԂɂāCtFXl𑝉.

	UINT64 fence = Device->FenceValue;
	HRESULT hr = Device->m_pCmdQue->Signal(Device->m_pFence, fence);

	// ҋ@.
	if (Device->m_pFence->GetCompletedValue() < fence) {
		hr = Device->m_pFence->SetEventOnCompletion(fence, Device->FenceEvent);
		WaitForSingleObject(Device->FenceEvent, INFINITE);
	}

	// t[obt@ԍXV.
	Device->FenceValue++;
	Device->FrameIndex = Device->m_pSwapChain->GetCurrentBackBufferIndex();
}

bool CDispWindow::Draw(CObjectManager* obj_manager, D3D12_CPU_DESCRIPTOR_HANDLE handle_rtv, D3D12_CPU_DESCRIPTOR_HANDLE handle_dsv) {

	Device->m_pCmdList1->ClearDepthStencilView(handle_dsv, D3D12_CLEAR_FLAG_DEPTH, 1.0f, 0, 0, nullptr);

	const float clearColor1[] = { 0.0f, 0.0f, 0.0f, 0.0f };
	Device->m_pCmdList1->ClearRenderTargetView(Device->handleToonRTV, clearColor1, 0, nullptr);
	Device->m_pCmdList1->ClearRenderTargetView(Device->handleToonDepthRTV, clearColor1, 0, nullptr);
	Device->m_pCmdList1->ClearRenderTargetView(Device->handleColliderRTV, clearColor1, 0, nullptr);

	const float clearColor2[] = { 0.8f, 0.8f, 0.8f, 1.0f };
	Device->m_pCmdList1->ClearRenderTargetView(handle_rtv, clearColor2, 0, nullptr);

	/////////////////////////////////////////////////////////////////
	//wi̕`

	// _[^[Qbg̐ݒ.
	Device->m_pCmdList1->OMSetRenderTargets(1, &handle_rtv, FALSE, &handle_dsv);
	Device->m_pCmdList1->SetPipelineState(Device->m_pPipeLineState1);

	obj_manager->DrawField(Device->m_pCmdList1);
	obj_manager->DrawObject(Device->m_pCmdList1);

	// _[^[Qbg̐ݒ.
	D3D12_CPU_DESCRIPTOR_HANDLE rtv[] = { Device->handleToonRTV, Device->handleToonDepthRTV };
	Device->m_pCmdList1->OMSetRenderTargets(2, rtv, FALSE, &handle_dsv);
	Device->m_pCmdList1->SetPipelineState(Device->m_pPipeLineState6);

	obj_manager->DrawCharacter(Device->m_pCmdList1);

	Device->m_pCmdList1->ClearDepthStencilView(handle_dsv, D3D12_CLEAR_FLAG_DEPTH, 1.0f, 0, 0, nullptr);

	if (obj_manager->DrawCollider == true) {

		Device->m_pCmdList1->OMSetRenderTargets(1, &Device->handleColliderRTV, FALSE, &handle_dsv);
		obj_manager->DrawCharacterCollider();
		obj_manager->DrawObjectCollider();
	}

	return true;
}

bool CDispWindow::DrawShadowMap( CObjectManager *obj_manager, D3D12_CPU_DESCRIPTOR_HANDLE handle_rtv, D3D12_CPU_DESCRIPTOR_HANDLE handle_dsv  ) {

	Device->m_pCmdList1->SetPipelineState(Device->m_pPipeLineState5);

	const float clearColor[] = { 0.0f, 0.0f, 0.0f, 0.0f };
	Device->m_pCmdList1->ClearRenderTargetView(Device->handleShadowMapRTV, clearColor, 0, nullptr);
	Device->m_pCmdList1->ClearDepthStencilView(handle_dsv, D3D12_CLEAR_FLAG_DEPTH, 1.0f, 0, 0, nullptr);

	// _[^[Qbg̐ݒ.
	Device->m_pCmdList1->OMSetRenderTargets(1, &Device->handleShadowMapRTV, FALSE, &handle_dsv);

	obj_manager->DrawField(Device->m_pCmdList1);
	obj_manager->DrawObject(Device->m_pCmdList1);
	obj_manager->DrawCharacter(Device->m_pCmdList1);

	return true;
}
